// noinspection TypeScriptUnresolvedVariable, ES6UnusedImports, JSUnusedLocalSymbols, TypeScriptCheckImport
import { DeepRequired } from "ts-essentials"
import { Maybe, IResponseListener, Endpoint } from "./types"

// Scalars
export type IDate = string | Date
export declare type ISODate = IDate

// Enums

export declare enum BookType {
  dolor = "DOLOR",
  ipsum = "IPSUM",
  sit = "SIT",
}

type AllEnums = BookType

export interface BooksWithOptionalParamsArgs {
  params?: BookSearchParamsAllOptional
}
export interface BooksWithRequiredParamsArgs {
  params: BookSearchParamsSomeRequired
}
export interface FailingQueryArgs {
  id: string
}

// Input/Output Types

/**
 * @deprecated Avoid directly using this interface. Instead, create a type alias based on the query/mutation return type.
 */

export interface Book {
  title?: string
  author?: string
  type?: BookType
  dateCreated?: ISODate
}

/**
 * @deprecated Avoid directly using this interface. Instead, create a type alias based on the query/mutation return type.
 */

export interface BookSearchParamsAllOptional {
  title?: string
  author?: string
  createdAfter?: ISODate
}

/**
 * @deprecated Avoid directly using this interface. Instead, create a type alias based on the query/mutation return type.
 */

export interface BookSearchParamsSomeRequired {
  title: string
  author?: string
}

/**
 * @deprecated Avoid directly using this interface. Instead, create a type alias based on the query/mutation return type.
 */

export interface Query {
  booksWithOptionalParams?: Book[]
  booksWithRequiredParams?: Book[]
  failingQuery?: string
}

// Selection Types

export interface BookSelection {
  title?: boolean
  author?: boolean
  type?: boolean
  dateCreated?: boolean
}

export interface BookSearchParamsAllOptionalSelection {
  title?: boolean
  author?: boolean
  createdAfter?: boolean
}

export interface BookSearchParamsSomeRequiredSelection {
  title?: boolean
  author?: boolean
}

export interface QuerySelection {
  booksWithOptionalParams?: {
    __headers?: { [key: string]: string }
    __retry?: boolean
    __alias?: string
    __args?: { params?: BookSearchParamsAllOptional }
  } & BookSelection
  booksWithRequiredParams?: {
    __headers?: { [key: string]: string }
    __retry?: boolean
    __alias?: string
    __args: { params: BookSearchParamsSomeRequired }
  } & BookSelection
  failingQuery?: {
    __headers?: { [key: string]: string }
    __retry?: boolean
    __alias?: string
    __args: { id: string }
  }
}

export declare const myApiClient: {
  addResponseListener: (listener: IResponseListener) => void
  setHeader: (key: string, value: string) => void
  setHeaders: (newHeaders: { [k: string]: string }) => void
  setUrl: (url: string) => void
  setRetryConfig: (options: {
    max: number
    waitBeforeRetry?: number
    before?: IResponseListener
  }) => void
  queries: {
    booksWithOptionalParams: Endpoint<
      {
        __headers?: { [key: string]: string }
        __retry?: boolean
        __alias?: string
        __args?: { params?: BookSearchParamsAllOptional }
      } & BookSelection,
      DeepRequired<Book[]>,
      AllEnums
    >
    booksWithRequiredParams: Endpoint<
      {
        __headers?: { [key: string]: string }
        __retry?: boolean
        __alias?: string
        __args: { params: BookSearchParamsSomeRequired }
      } & BookSelection,
      DeepRequired<Book[]>,
      AllEnums
    >
    failingQuery: Endpoint<
      {
        __headers?: { [key: string]: string }
        __retry?: boolean
        __alias?: string
        __args: { id: string }
      },
      string,
      AllEnums
    >
  }
  mutations: {}
}

export default myApiClient
